"""Fix sequences

Revision ID: af17b65998bc
Revises: 1107d03f17f8
Create Date: 2025-06-02 23:14:03.711430

"""

from alembic import op
import sqlalchemy as sa
import falcon_rest
import api_dnl
import imp, os


# revision identifiers, used by Alembic.
revision = 'af17b65998bc'
down_revision = '1107d03f17f8'
branch_labels = None
depends_on = None


def upgrade():
    upgrade_sql = """
BEGIN;

SELECT pg_catalog.setval('public.currency_currency_id_seq', 2, true);
SELECT pg_catalog.setval('public.route_strategy_route_strategy_id_seq', 1, true);
SELECT pg_catalog.setval('public.c4_livecall_user_id_seq', 1, true);
SELECT pg_catalog.setval('public.c4_lrn_id_seq', 1, true);
SELECT pg_catalog.setval('public.role_role_id_seq', 2, true);
SELECT pg_catalog.setval('public.voip_gateway_id_seq', 1, true);
SELECT pg_catalog.setval('public.rate_type_override_id_seq', 56, true);
SELECT pg_catalog.setval('public.sys_submenu_id_seq', 101, true);


END;
"""
    connection = op.get_bind()
    connection.execute(upgrade_sql)


def downgrade():
    # Arch users don't downgrade
    pass


"""
ADDONS
"""


def up_rec(table):
    """
    Update {table}_record schema
    """

    up_rec_sql = """
DO $do$
DECLARE
    rec_id INT := 1;
BEGIN
    -- Check if table exists
    IF NOT EXISTS (SELECT 0 FROM pg_tables WHERE tablename = '{table}') THEN
        RAISE WARNING 'Table {table} does not exist';
    ELSE
        -- Get last record_id
        SELECT record_id FROM {table}_record ORDER BY record_id DESC LIMIT 1
        INTO rec_id;
        -- Delete existing table
        DROP TRIGGER IF EXISTS class4_trig_record_{table} ON {table};
        DROP FUNCTION IF EXISTS class4_trigfun_record_{table};
        DROP TABLE IF EXISTS {table}_record;
        DROP SEQUENCE IF EXISTS {table}_record_record_id_seq;
    END IF;

    -- Create new table
    CREATE TABLE {table}_record (LIKE {table},
    "time" integer, flag character(1), record_id SERIAL PRIMARY KEY);
    ALTER TABLE {table}_record OWNER TO class4_user;
    CREATE INDEX IF NOT EXISTS {table}_record_time_idx
    ON {table}_record USING btree ("time");
    EXECUTE format(E'SELECT setval(\\'{table}_record_record_id_seq\\', %%L)',
    rec_id);

    -- Trigger
    CREATE FUNCTION class4_trigfun_record_{table}() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
        BEGIN
        IF (TG_OP = 'INSERT') THEN
            INSERT INTO public.{table}_record SELECT NEW.*,
            EXTRACT(EPOCH FROM current_timestamp(0)), 'I';
        ELSEIF (TG_OP = 'DELETE') THEN
            INSERT INTO public.{table}_record SELECT OLD.*,
            EXTRACT(EPOCH FROM current_timestamp(0)), 'D';
        ELSEIF (TG_OP = 'UPDATE') THEN
            INSERT INTO public.{table}_record SELECT OLD.*,
            EXTRACT(EPOCH FROM current_timestamp(0)), 'B';
            INSERT INTO public.{table}_record SELECT NEW.*,
            EXTRACT(EPOCH FROM current_timestamp(0)), 'A';
        END IF;
        return null;
        END;
    $$;
    ALTER FUNCTION class4_trigfun_record_{table}() OWNER TO class4_user;
    CREATE TRIGGER class4_trig_record_{table} AFTER INSERT OR DELETE OR UPDATE
    ON {table} FOR EACH ROW EXECUTE PROCEDURE class4_trigfun_record_{table}();
END $do$;
"""
    connection = op.get_bind()
    connection.execute(up_rec_sql.format_map(dict(table=table)))
